Package: hexbin
Version: 1.28.5
Title: Hexagonal Binning Routines
Authors@R: c(person(given = "Dan",
                      family = "Carr",
                      role = "aut",
                      email = "dcarr@voxel.galaxy.gmu.edu"),
               person(given = c("Nicholas"),
                      family = "Lewin-Koh",
                      role = "aut"),
               person(given = "Martin",
                      family = "Maechler",
                      role = "aut",
                      email = "maechler@stat.math.ethz.ch"),
               person(given = "Deepayan",
                      family = "Sarkar",
                      role = "aut",
                      email = "deepayan.sarkar@r-project.org"),
               person(given = "Edzer",
                      family = "Pebesma",
                      role = "cre",
                      comment = c(ORCID = "0000-0001-8049-7069"),
                      email = "edzer.pebesma@uni-muenster.de"))
Depends: R (>= 2.0.1), methods
Imports: lattice, grid, graphics, grDevices, stats, utils
Suggests: marray, affy, Biobase, limma, knitr
Description: Binning and plotting functions for hexagonal bins.
Collate: lattice.R BTC.R BTY.R grid.hexagons.R grid.hexlegend.R hbox.R
        hdiffplot.R hexbinList.R hexbinplot.R hexbin.s4.R hexpanel.R
        hexplom.R hexPlotMA.R hexutil.R hexViewport.R HO.R LINGRAY.R
        LOCS.R MAG.R RB.R smoothHexbin.R
License: GPL-2
VignetteBuilder: knitr
NeedsCompilation: yes
URL: https://github.com/edzer/hexbin
Packaged: 2024-11-13 17:43:55 UTC; edzer
Author: Dan Carr [aut],
  Nicholas Lewin-Koh [aut],
  Martin Maechler [aut],
  Deepayan Sarkar [aut],
  Edzer Pebesma [cre] (<https://orcid.org/0000-0001-8049-7069>)
Maintainer: Edzer Pebesma <edzer.pebesma@uni-muenster.de>
Repository: CRAN
Date/Publication: 2024-11-13 19:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:26:41 UTC; unix
Archs: hexbin.so.dSYM
